@@ -26,19 +26,23 @@ try:                  # Python 3
     from io import BytesIO
     from urllib.parse import parse_qs
     unicode = str
-    recode = lambda s: s.encode('iso-8859-1').decode('utf-8')
+
+    def recode(s):
+        return s.encode('iso-8859-1').decode('utf-8')
 except ImportError:   # Python 2
     from httplib import responses as HTTP_CODES
     from Cookie import SimpleCookie
     from cStringIO import StringIO as BytesIO
     from urlparse import parse_qs
     unicode = unicode
-    recode = lambda s: s.decode('utf-8')
+
+    def recode(s):
+        return s.decode('utf-8')
 
 DEFAULT_BIND = {'host': '127.0.0.1', 'port': 8080}
 MAIN_MODULE = '__main__'
 
-__version__ = '0.3'
+__version__ = '0.4'
 __all__ = ['HTTPError', 'BadRequest', 'Forbidden', 'NotFound',  # HTTP errors
            'MethodNotAllowed', 'InternalServerError', 'Redirect',
            # Base classes
@@ -165,15 +169,17 @@ def _format_vkw(value, kw, _quoted=re.compile(r"[^\w!#$%&'*.^`|~+-]").search):
 class lazyproperty(object):
     """A property whose value is computed only once."""
 
-    def __init__(self, function):
+    def __init__(self, function, func_name=None):
         self._function = function
         update_wrapper(self, function)
+        if func_name is not None:
+            self.__name__ = func_name
 
     def __get__(self, obj, cls=None):
         if obj is None:
             return self
         value = self._function(obj)
-        setattr(obj, self._function.__name__, value)
+        setattr(obj, self.__name__, value)
         return value
 
 
@@ -266,10 +272,10 @@ class Accept(object):
                 offer.startswith(mask + '-') or mask.startswith(offer + '-'))
 
     @classmethod
-    def header(cls, header):
+    def header(cls, header, func_name):
         def fget(request):
             return cls(header, request.headers[header])
-        return lazyproperty(fget)
+        return lazyproperty(fget, func_name)
 
 
 class HTTPHeaders(object):
@@ -356,9 +362,14 @@ class HTTPHeaders(object):
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, list(self))
 
-    keys = lambda self: [k for (k, v) in self]
-    values = lambda self: [v for (k, v) in self]
-    items = lambda self: list(self)
+    def keys(self):
+        return [k for (k, v) in self]
+
+    def values(self):
+        return [v for (k, v) in self]
+
+    def items(self):
+        return list(self)
 
 
 class EnvironHeaders(HTTPHeaders):
@@ -395,6 +406,7 @@ class Request(object):
         self.path = recode(environ.get('PATH_INFO', '/'))
         if self.path[-1:] != '/':
             self.path += '/'
+        self.script_name = environ.get('SCRIPT_NAME', '').rstrip('/')
         self.method = environ.get('REQUEST_METHOD', 'GET').upper()
         self.query = environ.get('QUERY_STRING', '')
         self.headers = EnvironHeaders(environ)
@@ -402,10 +414,10 @@ class Request(object):
             self.content_length = int(self.headers['Content-Length'] or 0)
         except ValueError:
             self.content_length = 0
-    accept = Accept.header('Accept')
-    accept_charset = Accept.header('Accept-Charset')
-    accept_encoding = Accept.header('Accept-Encoding')
-    accept_language = Accept.header('Accept-Language')
+    accept = Accept.header('Accept', 'accept')
+    accept_charset = Accept.header('Accept-Charset', 'accept_charset')
+    accept_encoding = Accept.header('Accept-Encoding', 'accept_encoding')
+    accept_language = Accept.header('Accept-Language', 'accept_language')
 
     def __getattr__(self, name):
         """Access the environment."""
@@ -426,6 +438,18 @@ class Request(object):
         return self.build_complex_dict()
     PUT = POST
 
+    @lazyproperty
+    def host_url(self):
+        """Build host URL."""
+        env = self.environ
+        scheme = env['wsgi.url_scheme']
+        host = env.get('HTTP_X_FORWARDED_HOST') or env.get('HTTP_HOST')
+        if not host:    # HTTP/1.0 client
+            (host, port) = (env['SERVER_NAME'], env['SERVER_PORT'])
+            if (scheme, port) not in (('https', '443'), ('http', '80')):
+                host += ':' + port
+        return scheme + '://' + host
+
     @lazyproperty
     def body(self):
         """Content of the request."""
@@ -479,6 +503,19 @@ class Request(object):
                 post_dict[field] = raw_data[field].value
         return post_dict
 
+    def get_url(self, path='', full=False):
+        """Build the absolute URL for an application path.
+
+        By default it builds the current request URL with leading and
+        trailing ``/`` and no query string.
+        The boolean argument ``full`` builds a full URL, incl. host.
+        """
+        if path[:1] != '/':
+            path = self.path + path
+        if full:
+            return self.host_url + self.script_name + path
+        return self.script_name + path
+
 
 class Response(object):
     charset = 'utf-8'
@@ -628,7 +665,7 @@ class Fiole(object):
     def encode_signed(self, name, value):
         """Return a signed string with timestamp."""
         value = base64.b64encode(tobytes(value)).decode('utf-8')
-        timestamp = '%X' % time.time()
+        timestamp = '%X' % int(time.time())
         signature = _create_signature(self.secret_key, name, value, timestamp)
         return (value + '|' + timestamp + '|' + signature)
 
@@ -732,8 +769,10 @@ put = _make_app_wrapper('put')
 delete = _make_app_wrapper('delete')
 errorhandler = _make_app_wrapper('errorhandler')
 
-#: Get the :class:`Fiole` application which is on the top of the stack.
-get_app = lambda: Fiole._stack[-1]
+
+def get_app():
+    """Get the :class:`Fiole` application which is on the top of the stack."""
+    return Fiole._stack[-1]
 default_app = Fiole.push()
 
 HTTP_CODES[418] = "I'm a teapot"                    # RFC 2324
@@ -1011,7 +1050,11 @@ class BlockBuilder(list):
             elif token == 'var':
                 filters = [f.strip() for f in value.split('|')]
                 value = filters.pop(0)
-                for f in filters + self.default_filters:
+                if filters and filters[-1] == 'n':
+                    filters.pop()
+                else:
+                    filters += self.default_filters
+                for f in filters:
                     value = self.filters.get(f, f) + '(' + value + ')'
             elif value:
                 value = repr(value)
